/* 
 * File:   interrupt_config_UART.h
 * Author: cstgeo
 *
 * Created on February 11, 2018, 2:22 PM
 */

#ifndef INTERRUPT_CONFIG_UART_H
#define	INTERRUPT_CONFIG_UART_H
#include "config.h"

/*
 * URXISEL<1:0>: Receive Interrupt Mode Selection bit
 * 11 = Reserved
 * 10 = Interrupt flag bit is asserted while receive buffer is 3/4 or more full (has 6 or more data characters)
 * 01 = Interrupt flag bit is asserted while receive buffer is 1/2 or more full (has 4 or more data characters)
 * 00 = Interrupt flag bit is asserted while receive buffer is not empty (has at least 1 data character)
 */
struct UART_Receive_Interrupt_Mode_tag {
    unsigned Buffer_Three_Quarters_Full:2;
    unsigned Buffer_Half_Full:2;
    unsigned Buffer_Not_Empty:2;
} UART_Receive_Interrupt_Mode = {0b10, 0b01, 0b00};

/*
 * UTXISEL<1:0>: TX Interrupt Mode Selection bits
 * 11 = Reserved, do not use
 * 10 = Interrupt is generated and asserted while the transmit buffer is empty
 * 01 = Interrupt is generated and asserted when all characters have been transmitted
 * 00 = Interrupt is generated and asserted while the transmit buffer contains at least one empty space
 */
struct UART_Transmit_Interrupt_Mode_tag {
    unsigned Buffer_Empty:2;
    unsigned Buffer_Just_Emptied:2;
    unsigned Buffer_Not_Full:2;
} UART_Transmit_Interrupt_Mode = {0b10, 0b01, 0b00};

// Interrupt Flag, Priority, and Enable bits
#define UART_1_Vector _UART_1_VECTOR
#define UART_1_Transmit_Interrupt_Flag IFS0bits.U1TXIF
#define UART_1_Transmit_Interrupt_Mode_Select U1STAbits.UTXISEL
#define UART_1_Receive_Interrupt_Flag IFS0bits.U1RXIF
#define UART_1_Receive_Interrupt_Mode_Select U1STAbits.URXISEL
#define UART_1_Error_Interrupt_Flag IFS0bits.U1EIF
#define UART_1_Parity_Error_Status U1STAbits.PERR
#define UART_1_Framing_Error_Status U1STAbits.FERR
#define UART_1_Receive_Buffer_Overflow_Status U1STAbits.OERR
#define UART_1_Interrupt_Priority IPC6bits.U1IP
#define UART_1_Interrupt_Subpriority IPC6bits.U1IS
#define UART_1_Transmit_Interrupt_Enable IEC0bits.U1TXIE
#define UART_1_Receive_Interrupt_Enable IEC0bits.U1RXIE
#define UART_1_Error_Interrupt_Enable IEC0bits.U1EIE

#define UART_2_Vector _UART_2_VECTOR
#define UART_2_Transmit_Interrupt_Flag IFS1bits.U2TXIF
#define UART_2_Transmit_Interrupt_Mode_Select U2STAbits.UTXISEL
#define UART_2_Receive_Interrupt_Flag IFS1bits.U2RXIF
#define UART_2_Receive_Interrupt_Mode_Select U2STAbits.URXISEL
#define UART_2_Error_Interrupt_Flag IFS1bits.U2EIF
#define UART_2_Parity_Error_Status U2STAbits.PERR
#define UART_2_Framing_Error_Status U2STAbits.FERR
#define UART_2_Receive_Buffer_Overflow_Status U2STAbits.OERR
#define UART_2_Interrupt_Priority IPC8bits.U2IP
#define UART_2_Interrupt_Subpriority IPC8bits.U2IS
#define UART_2_Transmit_Interrupt_Enable IEC1bits.U2TXIE
#define UART_2_Receive_Interrupt_Enable IEC1bits.U2RXIE
#define UART_2_Error_Interrupt_Enable IEC1bits.U2EIE

#define UART_6_Vector _UART_6_VECTOR
#define UART_6_Transmit_Interrupt_Flag IFS2bits.U6TXIF
#define UART_6_Transmit_Interrupt_Mode_Select U6STAbits.UTXISEL
#define UART_6_Receive_Interrupt_Flag IFS2bits.U6RXIF
#define UART_6_Receive_Interrupt_Mode_Select U6STAbits.URXISEL
#define UART_6_Error_Interrupt_Flag IFS2bits.U6EIF
#define UART_6_Parity_Error_Status U6STAbits.PERR
#define UART_6_Framing_Error_Status U6STAbits.FERR
#define UART_6_Receive_Buffer_Overflow_Status U6STAbits.OERR
#define UART_6_Interrupt_Priority IPC12bits.U6IP
#define UART_6_Interrupt_Subpriority IPC12bits.U6IS
#define UART_6_Transmit_Interrupt_Enable IEC2bits.U6TXIE

#define UART_6_Error_Interrupt_Enable IEC2bits.U6EIE

#define UART_Interrupt(object, full_name, event_type, priority, subpriority, tx_is_enabled, rx_is_enabled, err_is_enabled, tx_interrupt_mode, rx_interrupt_mode)\
 void object##_Interrupt_Settings() {\
  /*interrupt object##_##event_type##_Event autoIPL(priority) vector VECTOR(object);*/\
  object##_Interrupt_Priority = priority; /* Sets the priority for the interrupt */\
  object##_Interrupt_Subpriority = subpriority; /* Sets the subpriority for the interrupt */\
  object##_Transmit_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Transmit_Interrupt_Mode_Select = tx_interrupt_mode; /* Sets the rx interrupt mode */\
  object##_Receive_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Receive_Interrupt_Mode_Select = rx_interrupt_mode; /* Sets the rx interrupt mode */\
  object##_Error_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Transmit_Interrupt_Enable = tx_is_enabled; /* Sets the enable bit for the interrupt */\
  object##_Receive_Interrupt_Enable = rx_is_enabled; /* Sets the enable bit for the interrupt */\
  object##_Error_Interrupt_Enable = err_is_enabled; /* Sets the enable bit for the interrupt */\
 }\
 __attribute__((vector(VECTOR(full_name)),interrupt(PRIORITY(priority)), nomips16))\
 void object##_##event_type##_Event()

#endif	/* INTERRUPT_CONFIG_UART_H */

